package kc.mega.game;

import kc.mega.utils.Geom;
import kc.mega.utils.KUtils;

import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import robocode.AdvancedRobot;
import robocode.BulletHitEvent;
import robocode.HitByBulletEvent;
import robocode.HitRobotEvent;
import robocode.ScannedRobotEvent;


public class GameState {
  public static final int HISTORY_SIZE = 100;
  public static final int FUTURE_SIZE = 10;

  public final AdvancedRobot bot;
  public final HitRateTracker myHitRateTracker;
  public final HitRateTracker enemyHitRateTracker;

  public List<BotState> myHistory;
  public List<BotState> enemyHistory;
  public List<BotState> myFuture;
  public List<BotState> enemyFuture;
  public double lastEnemyBulletPower;
  public double enemyFirstShotBulletPower = 2;
  public boolean enemyFiredLastTick;
  public long gameTime;
  public int roundNum;
  public String enemyName;
  public long lastCollideTime;

  public long inactivityTicks;
  public double inactivityEnergy;

  public GameState(AdvancedRobot bot) {
    this.bot = bot;
    myHitRateTracker = new HitRateTracker();
    enemyHitRateTracker = new HitRateTracker();
  }

  public void init() {
    myHistory = new ArrayList<>();
    enemyHistory = new ArrayList<>();
    myFuture = new ArrayList<>();
    enemyFuture = new ArrayList<>();
    lastCollideTime = -1;
    lastEnemyBulletPower = -1;
    enemyFiredLastTick = false;
    gameTime = 0;
    inactivityEnergy = 0;
    inactivityTicks = 0;
    myHitRateTracker.init();
    enemyHitRateTracker.init();
    roundNum = bot.getRoundNum();
  }

  private BotState updateMyStates() {
    gameTime = bot.getTime();
    BotState prevMyState = getMyState();
    if (prevMyState != null && gameTime == prevMyState.gameTime) {
      return prevMyState;
    } else {
      BotState myState = new BotState(
          new Point2D.Double(bot.getX(), bot.getY()),
          bot.getHeadingRadians(),
          bot.getVelocity(),
          bot.getEnergy(),
          prevMyState == null ? bot.getGunHeat() :
            prevMyState.gunHeat - bot.getGunCoolingRate(),
            myHitRateTracker.getShots(),
          prevMyState == null ? 0 : prevMyState.lastFireTime,
          bot.getTime());
      updateStates(myState, myHistory);
      return myState;
    }
  }

  public void updateNoScan() {
    enemyFiredLastTick = false;
    updateMyStates();
    updateStates(getEnemyState().getNextState(0, 0), enemyHistory);
    setFutureStates();
  }

  public void update(ScannedRobotEvent e) {
    String[] split = e.getName().split(" ");
    enemyName = String.join(" ", Arrays.asList(split).subList(0, Math.min(2, split.length)));
    if (myHistory.size() > 0) {
      inactivityTicks++;
    } else {
      inactivityTicks = bot.getTime();
    }
    while (inactivityEnergy >= 10) {
      inactivityEnergy -= 10;
      inactivityTicks = 0;
    }

    BotState myState = updateMyStates();
    BotState prevEnemyState = getEnemyState();
    BotState enemyState = new BotState(
        Geom.project(myState.location,
            bot.getHeadingRadians() + e.getBearingRadians(), e.getDistance()),
        e.getHeadingRadians(),
        e.getVelocity(),
        e.getEnergy(),
        prevEnemyState == null ? bot.getGunHeat() :
          prevEnemyState.gunHeat - bot.getGunCoolingRate(),
        enemyHitRateTracker.getShots(),
        prevEnemyState == null ? 0 : prevEnemyState.lastFireTime,
        bot.getTime());
    updateStates(enemyState, enemyHistory);

    if (myHistory.size() == 1) {
      return;
    }

    double wallDamage = 0;
    if (enemyFuture.size() > 0 && Math.abs(enemyState.velocity) == 0 &&
        Math.abs(prevEnemyState.velocity) > 2 && BattleField.INSTANCE.nearWall(enemyState)) {
      wallDamage = Math.max(0, (Math.abs(enemyFuture.get(0).velocity) / 2) - 1);
    }
    double zapDamage = myHistory.size() < 2 ? 0 : myHistory.get(1).energy - myState.energy;
    double energyDifference = prevEnemyState.energy - enemyState.energy - wallDamage - zapDamage;
    enemyFiredLastTick = (energyDifference > (enemyState.energy == 0 ? 0 : 0.09999)) &&
        energyDifference < 3.0001 && prevEnemyState.gunHeat < 1e-4;
    if (enemyFiredLastTick) {
      if (lastEnemyBulletPower == -1) {
        enemyFirstShotBulletPower = energyDifference;
      }
      inactivityEnergy += energyDifference;
      lastEnemyBulletPower = energyDifference;
      prevEnemyState.updateFromFire(energyDifference);
      enemyState.gunHeat = prevEnemyState.gunHeat - bot.getGunCoolingRate();
      enemyState.lastFireTime = prevEnemyState.lastFireTime;
      enemyState.shotsFired = prevEnemyState.shotsFired;
      enemyHitRateTracker.logShot();
    }

    setFutureStates();
  }

  public void setFutureStates() {
    if (myFuture.size() > 0) {
      BotState myState = getMyState();
      BotState predictedState = myFuture.remove(0);
      if (Math.abs(myState.location.x - predictedState.location.x) +
          Math.abs(myState.location.y - predictedState.location.y) +
          Math.abs(myState.heading - predictedState.heading) +
          Math.abs(myState.velocity - predictedState.velocity) > 1e-4) {
        myFuture.clear();
      }
    }
    enemyFuture.clear();
    addPredictedFutureStates(myHistory, myFuture);
    addPredictedFutureStates(enemyHistory, enemyFuture);
  }

  public void addPredictedFutureStates(List<BotState> history, List<BotState> future) {
    int currentInd = future.size() - 1;
    int prevInd = future.size() - 2;
    BotState currentState = currentInd >= 0 ? future.get(currentInd) : history.get(-currentInd - 1);
    BotState prevState = prevInd >= 0 ? future.get(prevInd) : history.get(-prevInd - 1);
    while (future.size() < FUTURE_SIZE) {
      BotState nextState = currentState.predictNextState(prevState);
      future.add(nextState);
      prevState = currentState;
      currentState = nextState;
    }
  }

  public BotState getMyState() {
    return myHistory.size() > 0 ? myHistory.get(0): null;
  }

  public BotState getEnemyState() {
    return enemyHistory.size() > 0 ? enemyHistory.get(0): null;
  }

  public void updateStates(BotState newState, List<BotState> previousStates) {
    previousStates.add(0, newState);
    if (previousStates.size() > HISTORY_SIZE) {
      previousStates.remove(previousStates.size() - 1);
    }
  }

  public void onMyFire(double power) {
    inactivityEnergy += power;
    getMyState().updateFromFire(power);
    myHitRateTracker.logShot();
  }

  public void onBulletHit(BulletHitEvent e) {
    double power = e.getBullet().getPower();
    double damage = KUtils.bulletDamage(power, getEnemyState().energy);
    inactivityEnergy += damage;
    getEnemyState().updateFromHitByBullet(power);
    getMyState().updateFromBulletHit(power);
    myHitRateTracker.logHit(damage);
    if (getEnemyState().energy <= 0) {
      myHitRateTracker.logShotPassed(power);
    }
  }

  public void onHitByBullet(HitByBulletEvent e) {
    double power = e.getPower();
    double damage = KUtils.bulletDamage(power, getMyState().energy);
    inactivityEnergy += damage;
    getMyState().updateFromHitByBullet(power);
    getEnemyState().updateFromBulletHit(power);
    enemyHitRateTracker.logHit(damage);
    if (getMyState().energy <= 0) {
      enemyHitRateTracker.logShotPassed(e.getPower());
    }
  }

  public void onHitRobot(HitRobotEvent e) {
    if (myHistory.size() > 0) {
      getMyState().energy -= 0.6;
      getEnemyState().energy -= 0.6;
      inactivityEnergy += 1.2;
      if (e.isMyFault()) {
        lastCollideTime = bot.getTime();
      }
    }
  }

  public void setMyNextStates(List<BotState> states) {
    myFuture = states;
  }
}
