package kc.mega.learning;

import kc.mega.utils.KUtils;
import kc.mega.wave.WaveWithFeatures;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import ags.utils.KdTree;
import ags.utils.KdTree.Entry;
import jk.math.FastTrig;

public class WaveKNN<T> {
  private final KdTree<T> tree;
  private final String[] features;
  private final double[][] params;
  private final double distanceScale;
  private final int maxNeighbors;
  private final double neighborhoodSizeDivider;

  public static class Builder<T> {
    private String[] features;
    private double[][] params;
    private double distanceScale;
    private double neighborhoodSizeDivider;
    private int maxNeighbors;
    private int maxTreeSize = 50000;

    public Builder<T> features(String[] val) {features = val; return this;}
    public Builder<T> params(double[][] val) {params = val; return this;}
    public Builder<T> distanceScale(double val) {distanceScale = val; return this;}
    public Builder<T> neighborhoodSizeDivider(double val) {neighborhoodSizeDivider = val; return this;}
    public Builder<T> maxNeighbors(int val) {maxNeighbors = val; return this;}
    public Builder<T> maxTreeSize(int val) {maxTreeSize = val; return this;}
    public WaveKNN<T> build() {return new WaveKNN<T>(this);}
  }

  public WaveKNN(Builder<T> builder) {
    tree = new KdTree.Manhattan<>(builder.features.length, builder.maxTreeSize);
    this.features = builder.features;
    this.params = builder.params;
    this.distanceScale = builder.distanceScale;
    this.maxNeighbors = builder.maxNeighbors;
    this.neighborhoodSizeDivider = builder.neighborhoodSizeDivider;
  }

  public void addPoint(WaveWithFeatures w, T value) {
    tree.addPoint(embed(w), value);
  }

  public List<Entry<T>> getNeighbors(WaveWithFeatures w) {
    return tree.nearestNeighbor(embed(w), getNumNeighbors(), false);
  }

  public List<Entry<T>> getNeighbors(WaveWithFeatures w, int numNeighbors) {
    return tree.nearestNeighbor(embed(w), numNeighbors, false);
  }

  public int getNumNeighbors() {
    return Math.min(maxNeighbors, Math.max(5, (int)(tree.size() / neighborhoodSizeDivider)));
  }

  public boolean isEmpty() {
    return tree.size() > 0;
  }

  public double[] embed(WaveWithFeatures w) {
    return embed(getNormalizedFeatures(w));
  }

  private double[] embed(Map<String, Double> normalizedFeatures) {
    double[] embedding = new double[features.length];
    for (int i = 0; i < features.length; i++) {
      double featureVal = normalizedFeatures.get(features[i]);
      embedding[i] = params[i].length == 1 ? params[i][0] * featureVal :
        params[i][0] * Math.pow(1e-4 + params[i][1] + featureVal, params[i][2]);
    }
    return embedding;
  }

  public double[] getWeights(List<Entry<T>> neighbors) {
    double[] weights = new double[neighbors.size()];
    double maxLogit = Double.NEGATIVE_INFINITY;
    for (int i = 0; i < neighbors.size(); i++) {
      double distance = neighbors.get(i).distance;
      double logit = distance * distanceScale;
      weights[i] = logit;
      maxLogit = Math.max(logit, maxLogit);
    }
    KUtils.softmax(weights, maxLogit);
    return weights;
  }

  public static Map<String, Double> getNormalizedFeatures(WaveWithFeatures w) {
    double bft = w.distance / w.speed;
    double latVel = w.velocity * FastTrig.sin(w.relativeHeading);
    double advVel = w.velocity * FastTrig.cos(w.relativeHeading);
    double advDir = w.moveDirection * FastTrig.cos(w.relativeHeading);
    Map<String, Double> normalizedFeatures = new HashMap<>();
    normalizedFeatures.put("virtuality", w.virtuality / 5);
    normalizedFeatures.put("bft", bft / 100);
    normalizedFeatures.put("accel", Math.max(2 + w.accel, 0) / 2);
    normalizedFeatures.put("accelSign", Math.signum(w.accel));
    normalizedFeatures.put("latVel", Math.abs(latVel) / 8);
    normalizedFeatures.put("vel", Math.abs(w.velocity) / 8);
    normalizedFeatures.put("advVel", (advVel + 16) / 8);
    normalizedFeatures.put("advDir", (advDir + 1) / 2);
    normalizedFeatures.put("vChangeTimer", Math.min(w.vChangeTimer, 70) / bft);
    normalizedFeatures.put("dirChangeTimer", Math.min(w.dirChangeTimer, 70) / bft);
    normalizedFeatures.put("decelTimer", Math.min(w.decelTimer, 70) / bft);
    normalizedFeatures.put("distanceLast10", w.distanceLast10 / 80);
    normalizedFeatures.put("distanceLast20", w.distanceLast20 / 160);
    normalizedFeatures.put("orbitalWallAhead", w.orbitalWallAhead / 1.5);
    normalizedFeatures.put("orbitalWallReverse", w.orbitalWallReverse / 1.5);
    normalizedFeatures.put("maeWallAhead", w.maeWallAhead);
    normalizedFeatures.put("maeWallReverse", w.maeWallReverse);
    normalizedFeatures.put("stickWallAhead", w.stickWallAhead / (Math.PI / 2));
    normalizedFeatures.put("stickWallReverse", w.stickWallReverse / (Math.PI / 2));
    normalizedFeatures.put("gameTime", w.fireTime / 500.0);
    normalizedFeatures.put("shotsFired", w.shotsFired / 1000.0);
    normalizedFeatures.put("currentGF", (1 + w.currentGF) / 2);
    normalizedFeatures.put("didHit", w.didHit ? 1.0 : 0.0);
    normalizedFeatures.put("didCollide", w.didCollide ? 1.0 : 0.0);
    return normalizedFeatures;
  }
}
