package kc.mega.move;

import kc.mega.BeepBoop;
import kc.mega.game.BotState;
import kc.mega.game.GameState;
import kc.mega.game.HitRateTracker;
import kc.mega.misc.Strategy;
import kc.mega.wave.WaveManager;

import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.List;

import jk.math.FastTrig;
import robocode.AdvancedRobot;
import robocode.BulletHitBulletEvent;
import robocode.HitByBulletEvent;

public class Mover {
  private static final int VIRTUAL_TICKS_AHEAD = 2;

  private final AdvancedRobot bot;
  private final WaveManager.EnemyWaveManager waveManager;
  private final HitRateTracker hitRateTracker;
  private final PathSurfer surfer;
  private final DangerCalculator dangerComputer;

  private int nonHeadOnHits = 0;
  private MovementWave currentVirtualWave;

  public Mover(AdvancedRobot bot, Strategy strategy, WaveManager.EnemyWaveManager waveManager,
      HitRateTracker hitRateTracker) {
    this.bot = bot;
    this.waveManager = waveManager;
    this.hitRateTracker = hitRateTracker;
    dangerComputer = new DangerCalculator(strategy, hitRateTracker);
    surfer = new PathSurfer(bot, strategy, dangerComputer);
  }

  public void init() {
    currentVirtualWave = null;
    dangerComputer.init();
  }

  public void move(GameState gs) {
    if (gs.lastEnemyBulletPower > 0) {
      makeMovementWaves(gs);
    }
    for (MovementWave w : waveManager.updateWaves(gs.getMyState())) {
      if (w.hasBullet || w.didCollide) {
        hitRateTracker.logShotPassed(w.power);
        if (BeepBoop.writeData) {
          //datasetWriter.write("move-data-shots", w.toArray());
        }
      }
      if (bot.getOthers() > 0 && BeepBoop.writeData) {
        //datasetWriter.write("move-data-visits", w.toArray());
      }
      dangerComputer.onVisit(w, bot.getOthers() > 0);
    }
    dangerComputer.onTick(gs, waveManager.updatedShadowWaves);
    waveManager.updatedShadowWaves.clear();
    if (BeepBoop.paint) {
      dangerComputer.paint();
    }
    surfer.surf(gs, waveManager.getSurfableWaves(gs.getMyState()), BeepBoop.paint);
  }

  public void makeMovementWaves(GameState gs) {
    MovementWave w = (MovementWave)new MovementWave.Builder()
        .isVirtual(false)
        .waveManager(waveManager)
        .bins(dangerComputer.getBins())
        .myHistory(gs.enemyHistory.subList(1, gs.enemyHistory.size()))
        .enemyHistory(gs.myHistory.subList(1, gs.myHistory.size()))
        .power(gs.lastEnemyBulletPower)
        .hasBullet(gs.enemyFiredLastTick)
        .build();
    waveManager.add(w);
    if (w.hasBullet && currentVirtualWave != null) {
      waveManager.remove(currentVirtualWave);
      dangerComputer.remove(currentVirtualWave);
      currentVirtualWave = null;
    }
    double lastEnemyEnergy = gs.enemyHistory.get(1).energy;
    if (currentVirtualWave != null && (
        bot.getOthers() == 0 || lastEnemyEnergy < 0.099999)) {
      return;
    }
    List<BotState> myWaveStates = new ArrayList<>(gs.myHistory);
    List<BotState> enemyWaveStates = new ArrayList<>(gs.enemyHistory);
    double predictedBulletPower = Math.min(
        gs.lastEnemyBulletPower, lastEnemyEnergy < 0 ? 100 : lastEnemyEnergy);
    for (int i = 0; i < VIRTUAL_TICKS_AHEAD; i++) {
      BotState futureEnemyState = i == 0 ? gs.getEnemyState() : gs.enemyFuture.get(i - 1);
      if (futureEnemyState.gunHeat < 0.0001) {
        MovementWave virtualWave = (MovementWave)new MovementWave.Builder()
            .isVirtual(true)
            .waveManager(waveManager)
            .bins(dangerComputer.getBins())
            .myHistory(enemyWaveStates)
            .enemyHistory(myWaveStates)
            .power(predictedBulletPower)
            .build();
        waveManager.remove(currentVirtualWave);
        dangerComputer.remove(currentVirtualWave);
        waveManager.add(virtualWave);
        currentVirtualWave = virtualWave;
        break;
      }
      myWaveStates.add(0, gs.myFuture.get(i));
      enemyWaveStates.add(0, gs.enemyFuture.get(i));
    }
  }

  public void onHitByBullet(HitByBulletEvent e, GameState gs) {
    MovementWave hitWave = waveManager.getHitWave(e.getPower());
    if (hitWave != null && hitWave.fireTime < gs.gameTime - 1) {
      learnFromBullet(hitWave, e.getHeadingRadians());
    }
  }

  public void onBulletHitBullet(BulletHitBulletEvent e) {
    MovementWave hitWave = waveManager.getBulletHitBulletWave(
        new Point2D.Double(e.getHitBullet().getX(), e.getHitBullet().getY()),
        e.getHitBullet().getPower(), bot.getTime());
    if (hitWave != null) {
      learnFromBullet(hitWave, e.getHitBullet().getHeadingRadians());
    }
  }

  public void learnFromBullet(MovementWave wave, double bulletHeading) {
    wave.hitOffset = FastTrig.normalRelativeAngle(
        bulletHeading - wave.absoluteBearing);

    double hitGF = wave.hitGF();
    int hitBin = dangerComputer.getBins().hitBin(hitGF);
    if (BeepBoop.verbose && Math.abs(hitGF) < 0.99 && wave.didHit &&
        wave.shadows[hitBin] > 0.99) {
      System.out.println("Hit in bullet shadow");
    }

    if (Math.abs(wave.hitGF()) > 0.2 && ++nonHeadOnHits > (bot.getRoundNum() < 5 ? 0 : 1)) {
      dangerComputer.onNonHeadOnHit();
    }
    dangerComputer.onHit(wave);

    /*if (BeepBoop.writeData) {
      datasetWriter.write("move-data-hits", hitWave.asArray());
    }*/
  }

  public void printStats() {
    if (BeepBoop.verbose) {
      dangerComputer.printEstimatorWeights();
    }
    System.out.println("Enemy Hit Rate: " + hitRateTracker.getHitRateStr());
    if (BeepBoop.MC && !BeepBoop.MC2k7) {
      System.out.println("MC Score: " + (100 - (float)(
          hitRateTracker.getDamage() / (1 + bot.getRoundNum()))));
    }
  }
}
