package kc.mega.wave;

import kc.mega.aim.AimWave;
import kc.mega.game.BotState;
import kc.mega.game.PredictState;
import kc.mega.move.MovementWave;
import kc.mega.utils.Geom;

import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;


public class WaveManager {
  public final MyWaveManager myWaveManager = new MyWaveManager();
  public final EnemyWaveManager enemyWaveManager = new EnemyWaveManager();

  public void init() {
    myWaveManager.init();
    enemyWaveManager.init();
  }

  public void onBulletHitBullet() {
    for (MovementWave w : enemyWaveManager.getBulletWaves()) {
      w.shadows = new double[w.bins.nBins];
      enemyWaveManager.updatedShadowWaves.add(w);
      for (Bullet b : myWaveManager.getBullets()) {
        if (w.addBulletShadows(b, b.fireTime)) {
          enemyWaveManager.updatedShadowWaves.add(w);
        }
      }
    }
  }

  public void updateBulletShadows(Bullet b, List<MovementWave> waves) {
    for (MovementWave w : waves) {
      if (w.addBulletShadows(b, b.fireTime)) {
        enemyWaveManager.updatedShadowWaves.add(w);
      }
    }
  }

  public void updateBulletShadows(List<Bullet> bullets, MovementWave w) {
    for (Bullet b : bullets) {
      if (w.addBulletShadows(b, w.fireTime)) {
        enemyWaveManager.updatedShadowWaves.add(w);
      }
    }
  }

  public class MyWaveManager extends BotWaveManager<AimWave> {
    @Override
    public void add(AimWave w) {
      super.add(w);
      if (w.hasBullet) {
        updateBulletShadows(w.bullet, enemyWaveManager.getBulletWaves());
      }
    }

    public List<Bullet> getBullets() {
      return getBulletWaves().stream().map(w -> w.bullet).collect(Collectors.toList());
    }
  }

  public class EnemyWaveManager extends BotWaveManager<MovementWave> {
    public Set<MovementWave> updatedShadowWaves = new HashSet<>();

    @Override
    public void add(MovementWave w) {
      super.add(w);
      if (w.hasBullet && !w.isVirtual) {
        updateBulletShadows(myWaveManager.getBullets(), w);
      }
    }

    public List<MovementWave> getSurfableWaves(PredictState myState) {
      return waves.stream().filter(w -> w.hasBullet &&
          !w.didHit &&
          waveStatuses.get(w) <= Wave.BREAKING).sorted(
          // smaller impact time and larger power comes first
          (w1, w2) -> (int)Math.signum(
              w1.ticksUntilBreak - w2.ticksUntilBreak +
              (w2.power - w1.power) / 3.01)
          ).collect(Collectors.toList());
    }
  }

  public static abstract class BotWaveManager<W extends Wave> {
    protected List<W> waves;
    protected Map<W, Integer> waveStatuses;

    public void init() {
      waves = new ArrayList<>();
    }

    public void add(W w) {
      waves.add(w);
    }

    public void remove(W w) {
      waves.remove(w);
    }

    public List<W> getWaves() {
      return waves;
    }

    public List<W> updateWaves(BotState enemyState) {
      waveStatuses = new HashMap<>();
      List<W> passedWaves = new ArrayList<>();
      for (W w : new ArrayList<>(waves)) {
        int waveStatus = w.update(enemyState);
        waveStatuses.put(w, waveStatus);
        if (waveStatus == Wave.PASSED) {
          remove(w);
          if (w.hitOffset < 100) {
            passedWaves.add(w);
          } else {
            System.out.println("Wave passed with no data");
          }
        }
      }
      return passedWaves;
    }

    public Optional<W> closestWave(PredictState enemyState, boolean requireBullet) {
      return waves.stream()
          .filter(w -> w.getTicksUntilPasses(enemyState) > 0)
          .filter(w -> requireBullet ? w.hasBullet : true)
          .min((w1, w2) -> Math.abs(w1.getTicksUntilBreak(enemyState)) -
              Math.abs(w2.getTicksUntilBreak(enemyState)));
    }

    public double getCurrentGF(PredictState enemyState) {
      Optional<W> o = closestWave(enemyState, false);
      return o.isEmpty() ? 0 : o.get().getGF(Geom.offset(
          o.get().source, enemyState.location, o.get().absoluteBearing));
    }

    public W getHitWave(double power) {
      for (W w : getBulletWaves()) {
        if (waveStatuses.containsKey(w) && waveStatuses.get(w) > Wave.MIDAIR &&
            Math.abs(w.power - power) < 1e-4) {
          w.didHit = true;
          return w;
        }
      }
      System.out.println("Couldn't find bullet hit wave!");
      return null;
    }

    public W getBulletHitBulletWave(Point2D.Double hitLocation, double power, long gameTime) {
      for (W w : getBulletWaves()) {
        double distance = w.source.distance(hitLocation);
        if (Math.abs(w.power - power) < 1e-4 &&
            (Math.abs(distance - w.speed * (gameTime - w.fireTime - 1)) < 0.01 ||
             Math.abs(distance - w.speed * (gameTime - w.fireTime - 2)) < 0.01)) {
          w.hasBullet = false;
          w.didCollide = true;
          return w;
        }
      }
      System.out.println("Couldn't find bullet collision wave!");
      return null;
    }

    public List<W> getBulletWaves() {
      return waves.stream().filter(w -> w.hasBullet).collect(Collectors.toList());
    }
  }
}
