package kc.micro;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import robocode.AdvancedRobot;
import robocode.BulletHitEvent;
import robocode.Condition;
import robocode.HitByBulletEvent;
import robocode.Rules;
import robocode.ScannedRobotEvent;
import robocode.StatusEvent;
import robocode.util.Utils;

/**
* WaveShark, by Kevin Clark (Kev).
* Micro-sized bot with wave surfing
* See https://robowiki.net/wiki/WaveShark for more information.
*/
public class WaveShark extends AdvancedRobot {
  // Movement constants
  private static final int HIT_BINS = 31;
  private static final int MIDDLE_BIN = (HIT_BINS - 1) / 2;
  private static final double STICK_LENGTH = 160;

  // Gun constants
  private static final int MAX_BFT = 70;
  private static final int MAX_MATCH_LENGTH = 40;
  private static final double WALL_FEATURE = 0.5;
  private static final double ANTI_BULLET_SHIELD_FACTOR = 0.0005;

  // Energy management constants
  private static final int FULL_POWER_THRESHOLD = 120;
  private static final double BULLET_POWER = 1.9999;
  private static final double BULLET_DAMAGE = 10;
  private static final int BULLET_VELOCITY = 14;
  private static final int MIN_ENERGY = 2;

  static double enemyEnergy;
  static double lastVelocity;
  static double lastLastVelocity;
  static Point2D.Double myLocation;

  static double[][] hits = new double[5][HIT_BINS];
  static Wave surfWave;

  @Override
  public void run() {
    setAdjustRadarForGunTurn(true);
    setAdjustGunForRobotTurn(true);
    enemyEnergy = 102;
  }

  @Override
  public void onStatus(StatusEvent e) {
    setTurnRadarRightRadians(1);
  }

  @Override
  public void onScannedRobot(ScannedRobotEvent e) {
    myLocation = new Point2D.Double(getX(), getY());
    int direction_latVel_matchPos = -1;
    double energyDrop_absBearing_aimAngle;
    double distance;

    // Create a new enemy wave on energy drop
    Wave w = new Wave();
    if ((energyDrop_absBearing_aimAngle = enemyEnergy - (enemyEnergy = e.getEnergy())) > 0) {
      w.speed = Rules.getBulletSpeed(energyDrop_absBearing_aimAngle);
      w.hits = hits[(int)(Math.abs(lastLastVelocity)) >> 1];
      w.orbitDirection = Math.copySign(MIDDLE_BIN, lastLastVelocity);
      addCustomEvent(w);
    }
    lastLastVelocity = lastVelocity;
    w.source = projectMotion(energyDrop_absBearing_aimAngle = w.absoluteBearing =
        getHeadingRadians() + e.getBearingRadians(), distance = e.getDistance());

    // Predict going forward or backward until closest wave hits and pick the least dangerous option
    try {
      double maxRisk = Double.POSITIVE_INFINITY;
      double risk;
      Point2D.Double projectedLocation;
      while (direction_latVel_matchPos <= 1) {
        double targetHeading = energyDrop_absBearing_aimAngle + Math.toRadians(
            TARGET_HEADINGS.charAt(3 * (int)distance + direction_latVel_matchPos));
        while (fieldContains(targetHeading -= (direction_latVel_matchPos * 0.01),
            STICK_LENGTH * direction_latVel_matchPos) > 0);
        if ((risk =
            (1 + surfWave.hits[surfWave.hitBin(
                projectedLocation = projectMotion(
                    targetHeading,
                    (short)MOVE_DISTANCES.charAt(
                      17 * (surfWave.ticksUntilBreak + 3) +
                      (direction_latVel_matchPos * (int)(lastVelocity = getVelocity())))
                    * direction_latVel_matchPos)
                )]
             ) / projectedLocation.distance(w.source)) < maxRisk) {
          maxRisk = risk;
          setAhead(direction_latVel_matchPos << 5);
          setTurnRightRadians(Utils.normalRelativeAngle(targetHeading - getHeadingRadians()));
        }
        direction_latVel_matchPos += 2;
      }
    } catch (Exception ex) {}

    // Radar lock
    setTurnRadarRightRadians(2 * Utils.normalRelativeAngle(
        energyDrop_absBearing_aimAngle - getRadarHeadingRadians()));

    // Update log of enemy movements and pattern match
    int matchLen_bulletDist = MAX_MATCH_LENGTH;
    enemyHistory = String.valueOf(
        (char)(
            (direction_latVel_matchPos = (int)(e.getVelocity() *
                Math.sin(e.getHeadingRadians() - energyDrop_absBearing_aimAngle))) << 1
            | fieldContains(energyDrop_absBearing_aimAngle +
                Math.copySign(WALL_FEATURE, direction_latVel_matchPos), distance)
          )
        ).concat(enemyHistory);
    while((direction_latVel_matchPos = enemyHistory.indexOf(
        enemyHistory.substring(0, (--matchLen_bulletDist)), MAX_BFT)) < 0);

    // Fire and aim by playing forward enemy movements
    matchLen_bulletDist = (int)distance;
    if((int)(getEnergy()) > MIN_ENERGY) {
      setFire(BULLET_POWER + FULL_POWER_THRESHOLD / matchLen_bulletDist);
    }
    do {
       energyDrop_absBearing_aimAngle += ((short)enemyHistory.charAt(
           --direction_latVel_matchPos) >> 1) / distance;
    }
    while ((matchLen_bulletDist -= BULLET_VELOCITY) > 0);
    setTurnGunRightRadians(Utils.normalRelativeAngle(
        ANTI_BULLET_SHIELD_FACTOR + energyDrop_absBearing_aimAngle - getGunHeadingRadians()));

    // Each tick we re-select the closest active wave to surf
    surfWave = null;
  }

  public static int fieldContains(double heading, double distance) {
    return Integer.signum(new Rectangle2D.Double(18.0, 18.0, 764.0, 564.0).outcode(
        projectMotion(heading, distance)));
  }

  public static Point2D.Double projectMotion(double heading, double distance) {
    return new Point2D.Double(myLocation.x + (distance * Math.sin(heading)),
        myLocation.y + (distance * Math.cos(heading)));
  }

  @Override
  public void onBulletHit(BulletHitEvent e) {
    enemyEnergy -= BULLET_DAMAGE;
  }

  @Override
  public void onHitByBullet(HitByBulletEvent e) {
    try {
      int i = 0;
      do {
        surfWave.hits[i] += BIN_WEIGHTS.charAt((surfWave.hitBin(myLocation) << 5) + i++);
      } while(true);
    } catch (Exception ex) {}
  }

  public class Wave extends Condition {
    public Point2D.Double source;
    public double absoluteBearing;
    public double orbitDirection;
    public double speed;
    public int ticksSinceFire;
    public int ticksUntilBreak;
    public double[] hits;

    @Override
    public boolean test() {
      if((ticksUntilBreak = (int)(source.distance(myLocation) / speed) - (++ticksSinceFire)) > 0
          && surfWave == null) {
        surfWave = this;
      }
      return false;
    }

    public int hitBin(Point2D.Double location) {
      return MIDDLE_BIN + (int)(orbitDirection * Utils.normalRelativeAngle(
          absoluteBearing - Math.atan2(source.x - location.x, source.y - location.y)) /
          Math.asin(8 / speed));
    }
  }

  // log of enemy movement, initialize to something to avoid out-of-bounds
  static String enemyHistory = "" +
      (char)0 + (char)1 + (char)0 + (char)1 + (char)0 + (char)1 + (char)0 + (char)1 + (char)0 +
      (char)0 + (char)1 + (char)0 + (char)1 + (char)0 + (char)1 + (char)0 + (char)1 + (char)0 +
      (char)0 + (char)1 + (char)0 + (char)1 + (char)0 + (char)1 + (char)0 + (char)1 + (char)0 +
      (char)0 + (char)1 + (char)0 + (char)1 + (char)0 + (char)1 + (char)0 + (char)1 + (char)0 +
      (char)0 + (char)1 + (char)0 + (char)1 + (char)0 + (char)1 + (char)0 + (char)1 + (char)0 +
      (char)0 + (char)1 + (char)0 + (char)1 + (char)0 + (char)1 + (char)0 + (char)1 + (char)0 +
      (char)0 + (char)1 + (char)0 + (char)1 + (char)0 + (char)1 + (char)0 + (char)1 + (char)0 +
      (char)0 + (char)1 + (char)0 + (char)1 + (char)0 + (char)1 + (char)0 + (char)1 + (char)0 +
      (char)0 + (char)1 + (char)0 + (char)1 + (char)0 + (char)1 + (char)0 + (char)1 + (char)0 +
      (char)0 + (char)1 + (char)0 + (char)1 + (char)0 + (char)1 + (char)0 + (char)1 + (char)0;

  // how far we can move given our current velocity and ticks until the next wave hits us
  public static final String MOVE_DISTANCES = "" +
      (char)-6 + (char)-5 + (char)-4 + (char)-3 + (char)-2 + (char)-1 + (char)0 + (char)0 + (char)1 +
      (char)-6 + (char)-5 + (char)-4 + (char)-3 + (char)-2 + (char)-1 + (char)0 + (char)0 + (char)1 + (char)2 + (char)3 + (char)4 + (char)5 + (char)6 + (char)7 + (char)8 + (char)8 +
      (char)-6 + (char)-5 + (char)-4 + (char)-3 + (char)-2 + (char)-1 + (char)0 + (char)0 + (char)1 + (char)2 + (char)3 + (char)4 + (char)5 + (char)6 + (char)7 + (char)8 + (char)8 +
      (char)-6 + (char)-5 + (char)-4 + (char)-3 + (char)-2 + (char)-1 + (char)0 + (char)0 + (char)1 + (char)2 + (char)3 + (char)4 + (char)5 + (char)6 + (char)7 + (char)8 + (char)8 +
      (char)-6 + (char)-5 + (char)-4 + (char)-3 + (char)-2 + (char)-1 + (char)0 + (char)0 + (char)1 + (char)2 + (char)3 + (char)4 + (char)5 + (char)6 + (char)7 + (char)8 + (char)8 +
      (char)-6 + (char)-5 + (char)-4 + (char)-3 + (char)-2 + (char)-1 + (char)0 + (char)0 + (char)1 + (char)2 + (char)3 + (char)4 + (char)5 + (char)6 + (char)7 + (char)8 + (char)8 +
      (char)-10 + (char)-8 + (char)-6 + (char)-4 + (char)-2 + (char)0 + (char)1 + (char)2 + (char)3 + (char)5 + (char)7 + (char)9 + (char)11 + (char)13 + (char)15 + (char)16 + (char)16 +
      (char)-12 + (char)-9 + (char)-6 + (char)-3 + (char)-1 + (char)1 + (char)3 + (char)4 + (char)6 + (char)9 + (char)12 + (char)15 + (char)18 + (char)21 + (char)23 + (char)24 + (char)24 +
      (char)-12 + (char)-8 + (char)-5 + (char)-2 + (char)1 + (char)3 + (char)6 + (char)8 + (char)10 + (char)14 + (char)18 + (char)22 + (char)26 + (char)29 + (char)31 + (char)32 + (char)32 +
      (char)-11 + (char)-7 + (char)-3 + (char)0 + (char)4 + (char)7 + (char)10 + (char)12 + (char)15 + (char)20 + (char)25 + (char)30 + (char)34 + (char)37 + (char)39 + (char)40 + (char)40 +
      (char)-9 + (char)-4 + (char)0 + (char)4 + (char)8 + (char)11 + (char)15 + (char)18 + (char)21 + (char)27 + (char)33 + (char)38 + (char)42 + (char)45 + (char)47 + (char)48 + (char)48 +
      (char)-6 + (char)-1 + (char)4 + (char)8 + (char)13 + (char)17 + (char)21 + (char)24 + (char)28 + (char)35 + (char)41 + (char)46 + (char)50 + (char)53 + (char)55 + (char)56 + (char)56 +
      (char)-2 + (char)3 + (char)9 + (char)14 + (char)19 + (char)23 + (char)28 + (char)32 + (char)36 + (char)43 + (char)49 + (char)54 + (char)58 + (char)61 + (char)63 + (char)64 + (char)64 +
      (char)3 + (char)9 + (char)15 + (char)20 + (char)26 + (char)31 + (char)36 + (char)40 + (char)44 + (char)51 + (char)57 + (char)62 + (char)66 + (char)69 + (char)71 + (char)72 + (char)72 +
      (char)9 + (char)15 + (char)22 + (char)28 + (char)34 + (char)39 + (char)44 + (char)48 + (char)52 + (char)59 + (char)65 + (char)70 + (char)74 + (char)77 + (char)79 + (char)80 + (char)80 +
      (char)16 + (char)23 + (char)30 + (char)36 + (char)42 + (char)47 + (char)52 + (char)56 + (char)60 + (char)67 + (char)73 + (char)78 + (char)82 + (char)85 + (char)87 + (char)88 + (char)88 +
      (char)24 + (char)31 + (char)38 + (char)44 + (char)50 + (char)55 + (char)60 + (char)64 + (char)68 + (char)75 + (char)81 + (char)86 + (char)90 + (char)93 + (char)95 + (char)96 + (char)96 +
      (char)32 + (char)39 + (char)46 + (char)52 + (char)58 + (char)63 + (char)68 + (char)72 + (char)76 + (char)83 + (char)89 + (char)94 + (char)98 + (char)101 + (char)103 + (char)104 + (char)104 +
      (char)40 + (char)47 + (char)54 + (char)60 + (char)66 + (char)71 + (char)76 + (char)80 + (char)84 + (char)91 + (char)97 + (char)102 + (char)106 + (char)109 + (char)111 + (char)112 + (char)112 +
      (char)48 + (char)55 + (char)62 + (char)68 + (char)74 + (char)79 + (char)84 + (char)88 + (char)92 + (char)99 + (char)105 + (char)110 + (char)114 + (char)117 + (char)119 + (char)120 + (char)120 +
      (char)56 + (char)63 + (char)70 + (char)76 + (char)82 + (char)87 + (char)92 + (char)96 + (char)100 + (char)107 + (char)113 + (char)118 + (char)122 + (char)125 + (char)127 + (char)128 + (char)128 +
      (char)64 + (char)71 + (char)78 + (char)84 + (char)90 + (char)95 + (char)100 + (char)104 + (char)108 + (char)115 + (char)121 + (char)126 + (char)130 + (char)133 + (char)135 + (char)136 + (char)136 +
      (char)72 + (char)79 + (char)86 + (char)92 + (char)98 + (char)103 + (char)108 + (char)112 + (char)116 + (char)123 + (char)129 + (char)134 + (char)138 + (char)141 + (char)143 + (char)144 + (char)144 +
      (char)80 + (char)87 + (char)94 + (char)100 + (char)106 + (char)111 + (char)116 + (char)120 + (char)124 + (char)131 + (char)137 + (char)142 + (char)146 + (char)149 + (char)150 + (char)150 + (char)150 +
      (char)88 + (char)95 + (char)102 + (char)108 + (char)114 + (char)119 + (char)124 + (char)128 + (char)132 + (char)139 + (char)145 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)96 + (char)103 + (char)110 + (char)116 + (char)122 + (char)127 + (char)132 + (char)136 + (char)140 + (char)147 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)104 + (char)111 + (char)118 + (char)124 + (char)130 + (char)135 + (char)140 + (char)144 + (char)148 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)112 + (char)119 + (char)126 + (char)132 + (char)138 + (char)143 + (char)148 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)120 + (char)127 + (char)134 + (char)140 + (char)146 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)128 + (char)135 + (char)142 + (char)148 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)136 + (char)143 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)144 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 +
      (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150 + (char)150;

  // how much weight to add to a bin given the bin a bullet hit us in
  public static final String BIN_WEIGHTS = "" +
      (char)10000 + (char)5000 + (char)3333 + (char)2500 + (char)2000 + (char)1667 + (char)1429 + (char)1250 + (char)1111 + (char)1000 + (char)909 + (char)833 + (char)769 + (char)714 + (char)667 + (char)625 + (char)588 + (char)556 + (char)526 + (char)500 + (char)476 + (char)455 + (char)435 + (char)417 + (char)400 + (char)385 + (char)370 + (char)357 + (char)345 + (char)333 + (char)323 + (char)313 +
      (char)5000 + (char)10000 + (char)5000 + (char)3333 + (char)2500 + (char)2000 + (char)1667 + (char)1429 + (char)1250 + (char)1111 + (char)1000 + (char)909 + (char)833 + (char)769 + (char)714 + (char)667 + (char)625 + (char)588 + (char)556 + (char)526 + (char)500 + (char)476 + (char)455 + (char)435 + (char)417 + (char)400 + (char)385 + (char)370 + (char)357 + (char)345 + (char)333 + (char)323 +
      (char)3333 + (char)5000 + (char)10000 + (char)5000 + (char)3333 + (char)2500 + (char)2000 + (char)1667 + (char)1429 + (char)1250 + (char)1111 + (char)1000 + (char)909 + (char)833 + (char)769 + (char)714 + (char)667 + (char)625 + (char)588 + (char)556 + (char)526 + (char)500 + (char)476 + (char)455 + (char)435 + (char)417 + (char)400 + (char)385 + (char)370 + (char)357 + (char)345 + (char)333 +
      (char)2500 + (char)3333 + (char)5000 + (char)10000 + (char)5000 + (char)3333 + (char)2500 + (char)2000 + (char)1667 + (char)1429 + (char)1250 + (char)1111 + (char)1000 + (char)909 + (char)833 + (char)769 + (char)714 + (char)667 + (char)625 + (char)588 + (char)556 + (char)526 + (char)500 + (char)476 + (char)455 + (char)435 + (char)417 + (char)400 + (char)385 + (char)370 + (char)357 + (char)345 +
      (char)2000 + (char)2500 + (char)3333 + (char)5000 + (char)10000 + (char)5000 + (char)3333 + (char)2500 + (char)2000 + (char)1667 + (char)1429 + (char)1250 + (char)1111 + (char)1000 + (char)909 + (char)833 + (char)769 + (char)714 + (char)667 + (char)625 + (char)588 + (char)556 + (char)526 + (char)500 + (char)476 + (char)455 + (char)435 + (char)417 + (char)400 + (char)385 + (char)370 + (char)357 +
      (char)1667 + (char)2000 + (char)2500 + (char)3333 + (char)5000 + (char)10000 + (char)5000 + (char)3333 + (char)2500 + (char)2000 + (char)1667 + (char)1429 + (char)1250 + (char)1111 + (char)1000 + (char)909 + (char)833 + (char)769 + (char)714 + (char)667 + (char)625 + (char)588 + (char)556 + (char)526 + (char)500 + (char)476 + (char)455 + (char)435 + (char)417 + (char)400 + (char)385 + (char)370 +
      (char)1429 + (char)1667 + (char)2000 + (char)2500 + (char)3333 + (char)5000 + (char)10000 + (char)5000 + (char)3333 + (char)2500 + (char)2000 + (char)1667 + (char)1429 + (char)1250 + (char)1111 + (char)1000 + (char)909 + (char)833 + (char)769 + (char)714 + (char)667 + (char)625 + (char)588 + (char)556 + (char)526 + (char)500 + (char)476 + (char)455 + (char)435 + (char)417 + (char)400 + (char)385 +
      (char)1250 + (char)1429 + (char)1667 + (char)2000 + (char)2500 + (char)3333 + (char)5000 + (char)10000 + (char)5000 + (char)3333 + (char)2500 + (char)2000 + (char)1667 + (char)1429 + (char)1250 + (char)1111 + (char)1000 + (char)909 + (char)833 + (char)769 + (char)714 + (char)667 + (char)625 + (char)588 + (char)556 + (char)526 + (char)500 + (char)476 + (char)455 + (char)435 + (char)417 + (char)400 +
      (char)1111 + (char)1250 + (char)1429 + (char)1667 + (char)2000 + (char)2500 + (char)3333 + (char)5000 + (char)10000 + (char)5000 + (char)3333 + (char)2500 + (char)2000 + (char)1667 + (char)1429 + (char)1250 + (char)1111 + (char)1000 + (char)909 + (char)833 + (char)769 + (char)714 + (char)667 + (char)625 + (char)588 + (char)556 + (char)526 + (char)500 + (char)476 + (char)455 + (char)435 + (char)417 +
      (char)1000 + (char)1111 + (char)1250 + (char)1429 + (char)1667 + (char)2000 + (char)2500 + (char)3333 + (char)5000 + (char)10000 + (char)5000 + (char)3333 + (char)2500 + (char)2000 + (char)1667 + (char)1429 + (char)1250 + (char)1111 + (char)1000 + (char)909 + (char)833 + (char)769 + (char)714 + (char)667 + (char)625 + (char)588 + (char)556 + (char)526 + (char)500 + (char)476 + (char)455 + (char)435 +
      (char)909 + (char)1000 + (char)1111 + (char)1250 + (char)1429 + (char)1667 + (char)2000 + (char)2500 + (char)3333 + (char)5000 + (char)10000 + (char)5000 + (char)3333 + (char)2500 + (char)2000 + (char)1667 + (char)1429 + (char)1250 + (char)1111 + (char)1000 + (char)909 + (char)833 + (char)769 + (char)714 + (char)667 + (char)625 + (char)588 + (char)556 + (char)526 + (char)500 + (char)476 + (char)455 +
      (char)833 + (char)909 + (char)1000 + (char)1111 + (char)1250 + (char)1429 + (char)1667 + (char)2000 + (char)2500 + (char)3333 + (char)5000 + (char)10000 + (char)5000 + (char)3333 + (char)2500 + (char)2000 + (char)1667 + (char)1429 + (char)1250 + (char)1111 + (char)1000 + (char)909 + (char)833 + (char)769 + (char)714 + (char)667 + (char)625 + (char)588 + (char)556 + (char)526 + (char)500 + (char)476 +
      (char)769 + (char)833 + (char)909 + (char)1000 + (char)1111 + (char)1250 + (char)1429 + (char)1667 + (char)2000 + (char)2500 + (char)3333 + (char)5000 + (char)10000 + (char)5000 + (char)3333 + (char)2500 + (char)2000 + (char)1667 + (char)1429 + (char)1250 + (char)1111 + (char)1000 + (char)909 + (char)833 + (char)769 + (char)714 + (char)667 + (char)625 + (char)588 + (char)556 + (char)526 + (char)500 +
      (char)714 + (char)769 + (char)833 + (char)909 + (char)1000 + (char)1111 + (char)1250 + (char)1429 + (char)1667 + (char)2000 + (char)2500 + (char)3333 + (char)5000 + (char)10000 + (char)5000 + (char)3333 + (char)2500 + (char)2000 + (char)1667 + (char)1429 + (char)1250 + (char)1111 + (char)1000 + (char)909 + (char)833 + (char)769 + (char)714 + (char)667 + (char)625 + (char)588 + (char)556 + (char)526 +
      (char)667 + (char)714 + (char)769 + (char)833 + (char)909 + (char)1000 + (char)1111 + (char)1250 + (char)1429 + (char)1667 + (char)2000 + (char)2500 + (char)3333 + (char)5000 + (char)10000 + (char)5000 + (char)3333 + (char)2500 + (char)2000 + (char)1667 + (char)1429 + (char)1250 + (char)1111 + (char)1000 + (char)909 + (char)833 + (char)769 + (char)714 + (char)667 + (char)625 + (char)588 + (char)556 +
      (char)625 + (char)667 + (char)714 + (char)769 + (char)833 + (char)909 + (char)1000 + (char)1111 + (char)1250 + (char)1429 + (char)1667 + (char)2000 + (char)2500 + (char)3333 + (char)5000 + (char)10000 + (char)5000 + (char)3333 + (char)2500 + (char)2000 + (char)1667 + (char)1429 + (char)1250 + (char)1111 + (char)1000 + (char)909 + (char)833 + (char)769 + (char)714 + (char)667 + (char)625 + (char)588 +
      (char)588 + (char)625 + (char)667 + (char)714 + (char)769 + (char)833 + (char)909 + (char)1000 + (char)1111 + (char)1250 + (char)1429 + (char)1667 + (char)2000 + (char)2500 + (char)3333 + (char)5000 + (char)10000 + (char)5000 + (char)3333 + (char)2500 + (char)2000 + (char)1667 + (char)1429 + (char)1250 + (char)1111 + (char)1000 + (char)909 + (char)833 + (char)769 + (char)714 + (char)667 + (char)625 +
      (char)556 + (char)588 + (char)625 + (char)667 + (char)714 + (char)769 + (char)833 + (char)909 + (char)1000 + (char)1111 + (char)1250 + (char)1429 + (char)1667 + (char)2000 + (char)2500 + (char)3333 + (char)5000 + (char)10000 + (char)5000 + (char)3333 + (char)2500 + (char)2000 + (char)1667 + (char)1429 + (char)1250 + (char)1111 + (char)1000 + (char)909 + (char)833 + (char)769 + (char)714 + (char)667 +
      (char)526 + (char)556 + (char)588 + (char)625 + (char)667 + (char)714 + (char)769 + (char)833 + (char)909 + (char)1000 + (char)1111 + (char)1250 + (char)1429 + (char)1667 + (char)2000 + (char)2500 + (char)3333 + (char)5000 + (char)10000 + (char)5000 + (char)3333 + (char)2500 + (char)2000 + (char)1667 + (char)1429 + (char)1250 + (char)1111 + (char)1000 + (char)909 + (char)833 + (char)769 + (char)714 +
      (char)500 + (char)526 + (char)556 + (char)588 + (char)625 + (char)667 + (char)714 + (char)769 + (char)833 + (char)909 + (char)1000 + (char)1111 + (char)1250 + (char)1429 + (char)1667 + (char)2000 + (char)2500 + (char)3333 + (char)5000 + (char)10000 + (char)5000 + (char)3333 + (char)2500 + (char)2000 + (char)1667 + (char)1429 + (char)1250 + (char)1111 + (char)1000 + (char)909 + (char)833 + (char)769 +
      (char)476 + (char)500 + (char)526 + (char)556 + (char)588 + (char)625 + (char)667 + (char)714 + (char)769 + (char)833 + (char)909 + (char)1000 + (char)1111 + (char)1250 + (char)1429 + (char)1667 + (char)2000 + (char)2500 + (char)3333 + (char)5000 + (char)10000 + (char)5000 + (char)3333 + (char)2500 + (char)2000 + (char)1667 + (char)1429 + (char)1250 + (char)1111 + (char)1000 + (char)909 + (char)833 +
      (char)455 + (char)476 + (char)500 + (char)526 + (char)556 + (char)588 + (char)625 + (char)667 + (char)714 + (char)769 + (char)833 + (char)909 + (char)1000 + (char)1111 + (char)1250 + (char)1429 + (char)1667 + (char)2000 + (char)2500 + (char)3333 + (char)5000 + (char)10000 + (char)5000 + (char)3333 + (char)2500 + (char)2000 + (char)1667 + (char)1429 + (char)1250 + (char)1111 + (char)1000 + (char)909 +
      (char)435 + (char)455 + (char)476 + (char)500 + (char)526 + (char)556 + (char)588 + (char)625 + (char)667 + (char)714 + (char)769 + (char)833 + (char)909 + (char)1000 + (char)1111 + (char)1250 + (char)1429 + (char)1667 + (char)2000 + (char)2500 + (char)3333 + (char)5000 + (char)10000 + (char)5000 + (char)3333 + (char)2500 + (char)2000 + (char)1667 + (char)1429 + (char)1250 + (char)1111 + (char)1000 +
      (char)417 + (char)435 + (char)455 + (char)476 + (char)500 + (char)526 + (char)556 + (char)588 + (char)625 + (char)667 + (char)714 + (char)769 + (char)833 + (char)909 + (char)1000 + (char)1111 + (char)1250 + (char)1429 + (char)1667 + (char)2000 + (char)2500 + (char)3333 + (char)5000 + (char)10000 + (char)5000 + (char)3333 + (char)2500 + (char)2000 + (char)1667 + (char)1429 + (char)1250 + (char)1111 +
      (char)400 + (char)417 + (char)435 + (char)455 + (char)476 + (char)500 + (char)526 + (char)556 + (char)588 + (char)625 + (char)667 + (char)714 + (char)769 + (char)833 + (char)909 + (char)1000 + (char)1111 + (char)1250 + (char)1429 + (char)1667 + (char)2000 + (char)2500 + (char)3333 + (char)5000 + (char)10000 + (char)5000 + (char)3333 + (char)2500 + (char)2000 + (char)1667 + (char)1429 + (char)1250 +
      (char)385 + (char)400 + (char)417 + (char)435 + (char)455 + (char)476 + (char)500 + (char)526 + (char)556 + (char)588 + (char)625 + (char)667 + (char)714 + (char)769 + (char)833 + (char)909 + (char)1000 + (char)1111 + (char)1250 + (char)1429 + (char)1667 + (char)2000 + (char)2500 + (char)3333 + (char)5000 + (char)10000 + (char)5000 + (char)3333 + (char)2500 + (char)2000 + (char)1667 + (char)1429 +
      (char)370 + (char)385 + (char)400 + (char)417 + (char)435 + (char)455 + (char)476 + (char)500 + (char)526 + (char)556 + (char)588 + (char)625 + (char)667 + (char)714 + (char)769 + (char)833 + (char)909 + (char)1000 + (char)1111 + (char)1250 + (char)1429 + (char)1667 + (char)2000 + (char)2500 + (char)3333 + (char)5000 + (char)10000 + (char)5000 + (char)3333 + (char)2500 + (char)2000 + (char)1667 +
      (char)357 + (char)370 + (char)385 + (char)400 + (char)417 + (char)435 + (char)455 + (char)476 + (char)500 + (char)526 + (char)556 + (char)588 + (char)625 + (char)667 + (char)714 + (char)769 + (char)833 + (char)909 + (char)1000 + (char)1111 + (char)1250 + (char)1429 + (char)1667 + (char)2000 + (char)2500 + (char)3333 + (char)5000 + (char)10000 + (char)5000 + (char)3333 + (char)2500 + (char)2000 +
      (char)345 + (char)357 + (char)370 + (char)385 + (char)400 + (char)417 + (char)435 + (char)455 + (char)476 + (char)500 + (char)526 + (char)556 + (char)588 + (char)625 + (char)667 + (char)714 + (char)769 + (char)833 + (char)909 + (char)1000 + (char)1111 + (char)1250 + (char)1429 + (char)1667 + (char)2000 + (char)2500 + (char)3333 + (char)5000 + (char)10000 + (char)5000 + (char)3333 + (char)2500 +
      (char)333 + (char)345 + (char)357 + (char)370 + (char)385 + (char)400 + (char)417 + (char)435 + (char)455 + (char)476 + (char)500 + (char)526 + (char)556 + (char)588 + (char)625 + (char)667 + (char)714 + (char)769 + (char)833 + (char)909 + (char)1000 + (char)1111 + (char)1250 + (char)1429 + (char)1667 + (char)2000 + (char)2500 + (char)3333 + (char)5000 + (char)10000 + (char)5000 + (char)3333 +
      (char)323 + (char)333 + (char)345 + (char)357 + (char)370 + (char)385 + (char)400 + (char)417 + (char)435 + (char)455 + (char)476 + (char)500 + (char)526 + (char)556 + (char)588 + (char)625 + (char)667 + (char)714 + (char)769 + (char)833 + (char)909 + (char)1000 + (char)1111 + (char)1250 + (char)1429 + (char)1667 + (char)2000 + (char)2500 + (char)3333 + (char)5000 + (char)10000 + (char)5000;

  // what our bot's heading should be given our direction and distance to enemy
  public static final String TARGET_HEADINGS = "" +
      (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 +
      (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 +
      (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 +
      (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 +
      (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 +
      (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 +
      (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 +
      (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)15 + (char)0 + (char)165 + (char)23 + (char)0 + (char)158 +
      (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 +
      (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 +
      (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 +
      (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 +
      (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 +
      (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 +
      (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 +
      (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)23 + (char)0 + (char)158 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 +
      (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 +
      (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 +
      (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 +
      (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 +
      (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 +
      (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 +
      (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 +
      (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)45 + (char)0 + (char)135 + (char)61 + (char)0 + (char)119 + (char)61 + (char)0 + (char)119 + (char)61 +
      (char)0 + (char)119 + (char)62 + (char)0 + (char)118 + (char)62 + (char)0 + (char)118 + (char)62 + (char)0 + (char)118 + (char)62 + (char)0 + (char)118 + (char)62 + (char)0 + (char)118 + (char)62 + (char)0 +
      (char)118 + (char)62 + (char)0 + (char)118 + (char)62 + (char)0 + (char)118 + (char)62 + (char)0 + (char)118 + (char)62 + (char)0 + (char)118 + (char)62 + (char)0 + (char)118 + (char)62 + (char)0 + (char)118 +
      (char)62 + (char)0 + (char)118 + (char)62 + (char)0 + (char)118 + (char)63 + (char)0 + (char)117 + (char)63 + (char)0 + (char)117 + (char)63 + (char)0 + (char)117 + (char)63 + (char)0 + (char)117 + (char)63 +
      (char)0 + (char)117 + (char)63 + (char)0 + (char)117 + (char)63 + (char)0 + (char)117 + (char)63 + (char)0 + (char)117 + (char)63 + (char)0 + (char)117 + (char)63 + (char)0 + (char)117 + (char)63 + (char)0 +
      (char)117 + (char)63 + (char)0 + (char)117 + (char)63 + (char)0 + (char)117 + (char)64 + (char)0 + (char)116 + (char)64 + (char)0 + (char)116 + (char)64 + (char)0 + (char)116 + (char)64 + (char)0 + (char)116 +
      (char)64 + (char)0 + (char)116 + (char)64 + (char)0 + (char)116 + (char)64 + (char)0 + (char)116 + (char)64 + (char)0 + (char)116 + (char)64 + (char)0 + (char)116 + (char)64 + (char)0 + (char)116 + (char)64 +
      (char)0 + (char)116 + (char)64 + (char)0 + (char)116 + (char)64 + (char)0 + (char)116 + (char)64 + (char)0 + (char)116 + (char)65 + (char)0 + (char)115 + (char)65 + (char)0 + (char)115 + (char)65 + (char)0 +
      (char)115 + (char)65 + (char)0 + (char)115 + (char)65 + (char)0 + (char)115 + (char)65 + (char)0 + (char)115 + (char)65 + (char)0 + (char)115 + (char)65 + (char)0 + (char)115 + (char)65 + (char)0 + (char)115 +
      (char)65 + (char)0 + (char)115 + (char)65 + (char)0 + (char)115 + (char)65 + (char)0 + (char)115 + (char)65 + (char)0 + (char)115 + (char)65 + (char)0 + (char)115 + (char)66 + (char)0 + (char)114 + (char)66 +
      (char)0 + (char)114 + (char)66 + (char)0 + (char)114 + (char)66 + (char)0 + (char)114 + (char)66 + (char)0 + (char)114 + (char)66 + (char)0 + (char)114 + (char)66 + (char)0 + (char)114 + (char)66 + (char)0 +
      (char)114 + (char)66 + (char)0 + (char)114 + (char)66 + (char)0 + (char)114 + (char)66 + (char)0 + (char)114 + (char)66 + (char)0 + (char)114 + (char)66 + (char)0 + (char)114 + (char)66 + (char)0 + (char)114 +
      (char)67 + (char)0 + (char)113 + (char)67 + (char)0 + (char)113 + (char)67 + (char)0 + (char)113 + (char)67 + (char)0 + (char)113 + (char)67 + (char)0 + (char)113 + (char)67 + (char)0 + (char)113 + (char)67 +
      (char)0 + (char)113 + (char)67 + (char)0 + (char)113 + (char)67 + (char)0 + (char)113 + (char)67 + (char)0 + (char)113 + (char)67 + (char)0 + (char)113 + (char)67 + (char)0 + (char)113 + (char)67 + (char)0 +
      (char)113 + (char)67 + (char)0 + (char)113 + (char)68 + (char)0 + (char)112 + (char)68 + (char)0 + (char)112 + (char)68 + (char)0 + (char)112 + (char)68 + (char)0 + (char)112 + (char)68 + (char)0 + (char)112 +
      (char)68 + (char)0 + (char)112 + (char)68 + (char)0 + (char)112 + (char)68 + (char)0 + (char)112 + (char)68 + (char)0 + (char)112 + (char)68 + (char)0 + (char)112 + (char)68 + (char)0 + (char)112 + (char)68 +
      (char)0 + (char)112 + (char)68 + (char)0 + (char)112 + (char)68 + (char)0 + (char)112 + (char)69 + (char)0 + (char)111 + (char)69 + (char)0 + (char)111 + (char)69 + (char)0 + (char)111 + (char)69 + (char)0 +
      (char)111 + (char)69 + (char)0 + (char)111 + (char)69 + (char)0 + (char)111 + (char)69 + (char)0 + (char)111 + (char)69 + (char)0 + (char)111 + (char)69 + (char)0 + (char)111 + (char)69 + (char)0 + (char)111 +
      (char)69 + (char)0 + (char)111 + (char)69 + (char)0 + (char)111 + (char)69 + (char)0 + (char)111 + (char)69 + (char)0 + (char)111 + (char)70 + (char)0 + (char)110 + (char)70 + (char)0 + (char)110 + (char)70 +
      (char)0 + (char)110 + (char)70 + (char)0 + (char)110 + (char)70 + (char)0 + (char)110 + (char)70 + (char)0 + (char)110 + (char)70 + (char)0 + (char)110 + (char)70 + (char)0 + (char)110 + (char)70 + (char)0 +
      (char)110 + (char)70 + (char)0 + (char)110 + (char)70 + (char)0 + (char)110 + (char)70 + (char)0 + (char)110 + (char)70 + (char)0 + (char)110 + (char)70 + (char)0 + (char)110 + (char)71 + (char)0 + (char)109 +
      (char)71 + (char)0 + (char)109 + (char)71 + (char)0 + (char)109 + (char)71 + (char)0 + (char)109 + (char)71 + (char)0 + (char)109 + (char)71 + (char)0 + (char)109 + (char)71 + (char)0 + (char)109 + (char)71 +
      (char)0 + (char)109 + (char)71 + (char)0 + (char)109 + (char)71 + (char)0 + (char)109 + (char)71 + (char)0 + (char)109 + (char)71 + (char)0 + (char)109 + (char)71 + (char)0 + (char)109 + (char)71 + (char)0 +
      (char)109 + (char)72 + (char)0 + (char)108 + (char)72 + (char)0 + (char)108 + (char)72 + (char)0 + (char)108 + (char)72 + (char)0 + (char)108 + (char)72 + (char)0 + (char)108 + (char)72 + (char)0 + (char)108 +
      (char)72 + (char)0 + (char)108 + (char)72 + (char)0 + (char)108 + (char)72 + (char)0 + (char)108 + (char)72 + (char)0 + (char)108 + (char)72 + (char)0 + (char)108 + (char)72 + (char)0 + (char)108 + (char)72 +
      (char)0 + (char)108 + (char)72 + (char)0 + (char)108 + (char)73 + (char)0 + (char)107 + (char)73 + (char)0 + (char)107 + (char)73 + (char)0 + (char)107 + (char)73 + (char)0 + (char)107 + (char)73 + (char)0 +
      (char)107 + (char)73 + (char)0 + (char)107 + (char)73 + (char)0 + (char)107 + (char)73 + (char)0 + (char)107 + (char)73 + (char)0 + (char)107 + (char)73 + (char)0 + (char)107 + (char)73 + (char)0 + (char)107 +
      (char)73 + (char)0 + (char)107 + (char)73 + (char)0 + (char)107 + (char)73 + (char)0 + (char)107 + (char)74 + (char)0 + (char)106 + (char)74 + (char)0 + (char)106 + (char)74 + (char)0 + (char)106 + (char)74 +
      (char)0 + (char)106 + (char)74 + (char)0 + (char)106 + (char)74 + (char)0 + (char)106 + (char)74 + (char)0 + (char)106 + (char)74 + (char)0 + (char)106 + (char)74 + (char)0 + (char)106 + (char)74 + (char)0 +
      (char)106 + (char)74 + (char)0 + (char)106 + (char)74 + (char)0 + (char)106 + (char)74 + (char)0 + (char)106 + (char)74 + (char)0 + (char)106 + (char)75 + (char)0 + (char)105 + (char)75 + (char)0 + (char)105 +
      (char)75 + (char)0 + (char)105 + (char)75 + (char)0 + (char)105 + (char)75 + (char)0 + (char)105 + (char)75 + (char)0 + (char)105 + (char)75 + (char)0 + (char)105 + (char)75 + (char)0 + (char)105 + (char)75 +
      (char)0 + (char)105 + (char)75 + (char)0 + (char)105 + (char)75 + (char)0 + (char)105 + (char)75 + (char)0 + (char)105 + (char)75 + (char)0 + (char)105 + (char)75 + (char)0 + (char)105 + (char)76 + (char)0 +
      (char)104 + (char)76 + (char)0 + (char)104 + (char)76 + (char)0 + (char)104 + (char)76 + (char)0 + (char)104 + (char)76 + (char)0 + (char)104 + (char)76 + (char)0 + (char)104 + (char)76 + (char)0 + (char)104 +
      (char)76 + (char)0 + (char)104 + (char)76 + (char)0 + (char)104 + (char)76 + (char)0 + (char)104 + (char)76 + (char)0 + (char)104 + (char)76 + (char)0 + (char)104 + (char)76 + (char)0 + (char)104 + (char)76 +
      (char)0 + (char)104 + (char)77 + (char)0 + (char)103 + (char)77 + (char)0 + (char)103 + (char)77 + (char)0 + (char)103 + (char)77 + (char)0 + (char)103 + (char)77 + (char)0 + (char)103 + (char)77 + (char)0 +
      (char)103 + (char)77 + (char)0 + (char)103 + (char)77 + (char)0 + (char)103 + (char)77 + (char)0 + (char)103 + (char)77 + (char)0 + (char)103 + (char)77 + (char)0 + (char)103 + (char)77 + (char)0 + (char)103 +
      (char)77 + (char)0 + (char)103 + (char)77 + (char)0 + (char)103 + (char)78 + (char)0 + (char)102 + (char)78 + (char)0 + (char)102 + (char)78 + (char)0 + (char)102 + (char)78 + (char)0 + (char)102 + (char)78 +
      (char)0 + (char)102 + (char)78 + (char)0 + (char)102 + (char)78 + (char)0 + (char)102 + (char)78 + (char)0 + (char)102 + (char)78 + (char)0 + (char)102 + (char)78 + (char)0 + (char)102 + (char)78 + (char)0 +
      (char)102 + (char)78 + (char)0 + (char)102 + (char)78 + (char)0 + (char)102 + (char)78 + (char)0 + (char)102 + (char)79 + (char)0 + (char)101 + (char)79 + (char)0 + (char)101 + (char)79 + (char)0 + (char)101 +
      (char)79 + (char)0 + (char)101 + (char)79 + (char)0 + (char)101 + (char)79 + (char)0 + (char)101 + (char)79 + (char)0 + (char)101 + (char)79 + (char)0 + (char)101 + (char)79 + (char)0 + (char)101 + (char)79 +
      (char)0 + (char)101 + (char)79 + (char)0 + (char)101 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 +
      (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 +
      (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 +
      (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 +
      (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 +
      (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 +
      (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 +
      (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 +
      (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 +
      (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 +
      (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 +
      (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 +
      (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 +
      (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 +
      (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 +
      (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 +
      (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 +
      (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 +
      (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 +
      (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 +
      (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 +
      (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 +
      (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 +
      (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 +
      (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 +
      (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 +
      (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 +
      (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 +
      (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 +
      (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 +
      (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 +
      (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 +
      (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 +
      (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 +
      (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 +
      (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 +
      (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 +
      (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 +
      (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 +
      (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)91 + (char)0 + (char)89 + (char)89 + (char)0 + (char)91 + (char)97 + (char)0 + (char)83 + (char)97 + (char)0 + (char)83 + (char)97 + (char)0 +
      (char)83 + (char)97 + (char)0 + (char)83 + (char)97 + (char)0 + (char)83 + (char)98 + (char)0 + (char)82 + (char)98 + (char)0 + (char)82 + (char)98 + (char)0 + (char)82 + (char)98 + (char)0 + (char)82 +
      (char)98 + (char)0 + (char)82 + (char)98 + (char)0 + (char)82 + (char)98 + (char)0 + (char)82 + (char)98 + (char)0 + (char)82 + (char)98 + (char)0 + (char)82 + (char)98 + (char)0 + (char)82 + (char)98 +
      (char)0 + (char)82 + (char)98 + (char)0 + (char)82 + (char)98 + (char)0 + (char)82 + (char)98 + (char)0 + (char)82 + (char)99 + (char)0 + (char)81 + (char)99 + (char)0 + (char)81 + (char)99 + (char)0 +
      (char)81 + (char)99 + (char)0 + (char)81 + (char)99 + (char)0 + (char)81 + (char)99 + (char)0 + (char)81 + (char)99 + (char)0 + (char)81 + (char)99 + (char)0 + (char)81 + (char)99 + (char)0 + (char)81 +
      (char)99 + (char)0 + (char)81 + (char)99 + (char)0 + (char)81 + (char)99 + (char)0 + (char)81 + (char)99 + (char)0 + (char)81 + (char)99 + (char)0 + (char)81 + (char)100 + (char)0 + (char)80 + (char)100 +
      (char)0 + (char)80 + (char)100 + (char)0 + (char)80 + (char)100 + (char)0 + (char)80 + (char)100 + (char)0 + (char)80 + (char)100 + (char)0 + (char)80 + (char)100 + (char)0 + (char)80 + (char)100 + (char)0 +
      (char)80 + (char)100 + (char)0 + (char)80 + (char)100 + (char)0 + (char)80 + (char)100 + (char)0 + (char)80 + (char)100 + (char)0 + (char)80 + (char)100 + (char)0 + (char)80 + (char)100 + (char)0 + (char)80 +
      (char)101 + (char)0 + (char)79 + (char)101 + (char)0 + (char)79 + (char)101 + (char)0 + (char)79 + (char)101 + (char)0 + (char)79 + (char)101 + (char)0 + (char)79 + (char)101 + (char)0 + (char)79 + (char)101 +
      (char)0 + (char)79 + (char)101 + (char)0 + (char)79 + (char)101 + (char)0 + (char)79 + (char)101 + (char)0 + (char)79 + (char)101 + (char)0 + (char)79 + (char)101 + (char)0 + (char)79 + (char)101 + (char)0 +
      (char)79 + (char)101 + (char)0 + (char)79 + (char)102 + (char)0 + (char)78 + (char)102 + (char)0 + (char)78 + (char)102 + (char)0 + (char)78 + (char)102 + (char)0 + (char)78 + (char)102 + (char)0 + (char)78 +
      (char)102 + (char)0 + (char)78 + (char)102 + (char)0 + (char)78 + (char)102 + (char)0 + (char)78 + (char)102 + (char)0 + (char)78 + (char)102 + (char)0 + (char)78 + (char)102 + (char)0 + (char)78 + (char)102 +
      (char)0 + (char)78 + (char)102 + (char)0 + (char)78 + (char)102 + (char)0 + (char)78 + (char)103 + (char)0 + (char)77 + (char)103 + (char)0 + (char)77 + (char)103 + (char)0 + (char)77 + (char)103 + (char)0 +
      (char)77 + (char)103 + (char)0 + (char)77 + (char)103 + (char)0 + (char)77 + (char)103 + (char)0 + (char)77 + (char)103 + (char)0 + (char)77 + (char)103 + (char)0 + (char)77 + (char)103 + (char)0 + (char)77 +
      (char)103 + (char)0 + (char)77 + (char)103 + (char)0 + (char)77 + (char)103 + (char)0 + (char)77 + (char)103 + (char)0 + (char)77 + (char)104 + (char)0 + (char)76 + (char)104 + (char)0 + (char)76 + (char)104 +
      (char)0 + (char)76 + (char)104 + (char)0 + (char)76 + (char)104 + (char)0 + (char)76 + (char)104 + (char)0 + (char)76 + (char)104 + (char)0 + (char)76 + (char)104 + (char)0 + (char)76 + (char)104 + (char)0 +
      (char)76 + (char)104 + (char)0 + (char)76 + (char)104 + (char)0 + (char)76 + (char)104 + (char)0 + (char)76 + (char)104 + (char)0 + (char)76 + (char)104 + (char)0 + (char)76 + (char)105 + (char)0 + (char)75 +
      (char)105 + (char)0 + (char)75 + (char)105 + (char)0 + (char)75 + (char)105 + (char)0 + (char)75 + (char)105 + (char)0 + (char)75 + (char)105 + (char)0 + (char)75 + (char)105 + (char)0 + (char)75 + (char)105 +
      (char)0 + (char)75 + (char)105 + (char)0 + (char)75 + (char)105 + (char)0 + (char)75 + (char)105 + (char)0 + (char)75 + (char)105 + (char)0 + (char)75 + (char)105 + (char)0 + (char)75 + (char)105 + (char)0 +
      (char)75 + (char)106 + (char)0 + (char)74 + (char)106 + (char)0 + (char)74 + (char)106 + (char)0 + (char)74 + (char)106 + (char)0 + (char)74 + (char)106 + (char)0 + (char)74 + (char)106 + (char)0 + (char)74 +
      (char)106 + (char)0 + (char)74 + (char)106 + (char)0 + (char)74 + (char)106 + (char)0 + (char)74 + (char)106 + (char)0 + (char)74 + (char)106 + (char)0 + (char)74 + (char)106 + (char)0 + (char)74 + (char)106 +
      (char)0 + (char)74 + (char)106 + (char)0 + (char)74 + (char)107 + (char)0 + (char)73 + (char)107 + (char)0 + (char)73 + (char)107 + (char)0 + (char)73 + (char)107 + (char)0 + (char)73 + (char)107 + (char)0 +
      (char)73 + (char)107 + (char)0 + (char)73 + (char)107 + (char)0 + (char)73 + (char)107 + (char)0 + (char)73 + (char)107 + (char)0 + (char)73 + (char)107 + (char)0 + (char)73 + (char)107 + (char)0 + (char)73 +
      (char)107 + (char)0 + (char)73 + (char)107 + (char)0 + (char)73 + (char)107 + (char)0 + (char)73 + (char)108 + (char)0 + (char)72 + (char)108 + (char)0 + (char)72 + (char)108 + (char)0 + (char)72 + (char)108 +
      (char)0 + (char)72 + (char)108 + (char)0 + (char)72 + (char)108 + (char)0 + (char)72 + (char)108 + (char)0 + (char)72 + (char)108 + (char)0 + (char)72 + (char)108 + (char)0 + (char)72 + (char)108 + (char)0 +
      (char)72 + (char)108 + (char)0 + (char)72 + (char)108 + (char)0 + (char)72 + (char)108 + (char)0 + (char)72 + (char)108 + (char)0 + (char)72 + (char)109 + (char)0 + (char)71 + (char)109 + (char)0 + (char)71 +
      (char)109 + (char)0 + (char)71 + (char)109 + (char)0 + (char)71 + (char)109 + (char)0 + (char)71 + (char)109 + (char)0 + (char)71 + (char)109 + (char)0 + (char)71 + (char)109 + (char)0 + (char)71 + (char)109 +
      (char)0 + (char)71 + (char)109 + (char)0 + (char)71 + (char)109 + (char)0 + (char)71 + (char)109 + (char)0 + (char)71 + (char)109 + (char)0 + (char)71 + (char)109 + (char)0 + (char)71 + (char)110 + (char)0 +
      (char)70 + (char)110 + (char)0 + (char)70 + (char)110 + (char)0 + (char)70 + (char)110 + (char)0 + (char)70 + (char)110 + (char)0 + (char)70 + (char)110 + (char)0 + (char)70 + (char)110 + (char)0 + (char)70 +
      (char)110 + (char)0 + (char)70 + (char)110 + (char)0 + (char)70 + (char)110 + (char)0 + (char)70 + (char)110 + (char)0 + (char)70 + (char)110 + (char)0 + (char)70 + (char)110 + (char)0 + (char)70 + (char)110 +
      (char)0 + (char)70 + (char)111 + (char)0 + (char)69 + (char)111 + (char)0 + (char)69 + (char)111 + (char)0 + (char)69 + (char)111 + (char)0 + (char)69 + (char)111 + (char)0 + (char)69 + (char)111 + (char)0 +
      (char)69 + (char)111 + (char)0 + (char)69 + (char)111 + (char)0 + (char)69 + (char)111 + (char)0 + (char)69 + (char)111 + (char)0 + (char)69 + (char)111 + (char)0 + (char)69 + (char)111 + (char)0 + (char)69 +
      (char)111 + (char)0 + (char)69 + (char)111 + (char)0 + (char)69 + (char)112 + (char)0 + (char)68 + (char)112 + (char)0 + (char)68 + (char)112 + (char)0 + (char)68 + (char)112 + (char)0 + (char)68 + (char)112 +
      (char)0 + (char)68 + (char)112 + (char)0 + (char)68 + (char)112 + (char)0 + (char)68 + (char)112 + (char)0 + (char)68 + (char)112 + (char)0 + (char)68 + (char)112 + (char)0 + (char)68 + (char)112 + (char)0 +
      (char)68 + (char)112 + (char)0 + (char)68 + (char)112 + (char)0 + (char)68 + (char)112 + (char)0 + (char)68 + (char)113 + (char)0 + (char)67 + (char)113 + (char)0 + (char)67 + (char)113 + (char)0 + (char)67 +
      (char)113 + (char)0 + (char)67 + (char)113 + (char)0 + (char)67 + (char)113 + (char)0 + (char)67 + (char)113 + (char)0 + (char)67 + (char)113 + (char)0 + (char)67 + (char)113 + (char)0 + (char)67 + (char)113 +
      (char)0 + (char)67 + (char)113 + (char)0 + (char)67 + (char)113 + (char)0 + (char)67 + (char)113 + (char)0 + (char)67 + (char)113 + (char)0 + (char)67 + (char)114 + (char)0 + (char)66 + (char)114 + (char)0 +
      (char)66 + (char)114 + (char)0 + (char)66 + (char)114 + (char)0 + (char)66 + (char)114 + (char)0 + (char)66 + (char)114 + (char)0 + (char)66 + (char)114 + (char)0 + (char)66 + (char)114 + (char)0 + (char)66 +
      (char)114 + (char)0 + (char)66 + (char)114 + (char)0 + (char)66 + (char)114 + (char)0 + (char)66 + (char)114 + (char)0 + (char)66 + (char)114 + (char)0 + (char)66 + (char)114 + (char)0 + (char)66 + (char)115 +
      (char)0 + (char)65 + (char)115 + (char)0 + (char)65 + (char)115 + (char)0 + (char)65 + (char)115 + (char)0 + (char)65 + (char)115 + (char)0 + (char)65 + (char)115 + (char)0 + (char)65 + (char)115 + (char)0 +
      (char)65 + (char)115 + (char)0 + (char)65 + (char)115 + (char)0 + (char)65 + (char)115 + (char)0 + (char)65 + (char)115 + (char)0 + (char)65 + (char)115 + (char)0 + (char)65 + (char)115 + (char)0 + (char)65 +
      (char)115 + (char)0 + (char)65 + (char)116 + (char)0 + (char)64 + (char)116 + (char)0 + (char)64 + (char)116 + (char)0 + (char)64 + (char)116 + (char)0 + (char)64 + (char)116 + (char)0 + (char)64 + (char)116 +
      (char)0 + (char)64 + (char)116 + (char)0 + (char)64 + (char)116 + (char)0 + (char)64 + (char)116 + (char)0 + (char)64 + (char)116 + (char)0 + (char)64 + (char)116 + (char)0 + (char)64 + (char)116 + (char)0 +
      (char)64 + (char)116 + (char)0 + (char)64 + (char)116 + (char)0 + (char)64 + (char)117 + (char)0 + (char)63 + (char)117 + (char)0 + (char)63 + (char)117 + (char)0 + (char)63 + (char)117 + (char)0 + (char)63 +
      (char)117 + (char)0 + (char)63 + (char)117 + (char)0 + (char)63 + (char)117 + (char)0 + (char)63 + (char)117 + (char)0 + (char)63 + (char)117 + (char)0 + (char)63 + (char)117 + (char)0 + (char)63 + (char)117 +
      (char)0 + (char)63 + (char)117 + (char)0 + (char)63 + (char)117 + (char)0 + (char)63 + (char)118 + (char)0 + (char)62 + (char)118 + (char)0 + (char)62 + (char)118 + (char)0 + (char)62 + (char)118 + (char)0 +
      (char)62 + (char)118 + (char)0 + (char)62 + (char)118 + (char)0 + (char)62 + (char)118 + (char)0 + (char)62 + (char)118 + (char)0 + (char)62 + (char)118 + (char)0 + (char)62 + (char)118 + (char)0 + (char)62 +
      (char)118 + (char)0 + (char)62 + (char)118 + (char)0 + (char)62 + (char)118 + (char)0 + (char)62 + (char)118 + (char)0 + (char)62 + (char)119 + (char)0 + (char)61 + (char)119 + (char)0 + (char)61 + (char)119 +
      (char)0 + (char)61 + (char)119 + (char)0 + (char)61 + (char)119 + (char)0 + (char)61 + (char)119 + (char)0 + (char)61 + (char)119 + (char)0 + (char)61 + (char)119 + (char)0 + (char)61 + (char)119 + (char)0 +
      (char)61 + (char)119 + (char)0 + (char)61 + (char)119 + (char)0 + (char)61 + (char)119 + (char)0 + (char)61 + (char)119 + (char)0 + (char)61 + (char)119 + (char)0 + (char)61 + (char)120 + (char)0 + (char)60 +
      (char)120 + (char)0 + (char)60 + (char)120 + (char)0 + (char)60 + (char)120 + (char)0 + (char)60 + (char)120 + (char)0 + (char)60 + (char)120 + (char)0 + (char)60 + (char)120 + (char)0 + (char)60 + (char)120 +
      (char)0 + (char)60 + (char)120 + (char)0 + (char)60 + (char)120 + (char)0 + (char)60 + (char)120 + (char)0 + (char)60 + (char)120 + (char)0 + (char)60 + (char)120 + (char)0 + (char)60 + (char)120 + (char)0 +
      (char)60 + (char)121 + (char)0 + (char)59 + (char)121 + (char)0 + (char)59 + (char)121 + (char)0 + (char)59 + (char)121 + (char)0 + (char)59 + (char)121 + (char)0 + (char)59 + (char)121 + (char)0 + (char)59 +
      (char)121 + (char)0 + (char)59 + (char)121 + (char)0 + (char)59 + (char)121 + (char)0 + (char)59 + (char)121 + (char)0 + (char)59 + (char)121 + (char)0 + (char)59 + (char)121 + (char)0 + (char)59 + (char)121 +
      (char)0 + (char)59 + (char)121 + (char)0 + (char)59 + (char)122 + (char)0 + (char)58 + (char)122 + (char)0 + (char)58 + (char)122 + (char)0 + (char)58 + (char)122 + (char)0 + (char)58 + (char)122 + (char)0 +
      (char)58 + (char)122 + (char)0 + (char)58 + (char)122 + (char)0 + (char)58 + (char)122 + (char)0 + (char)58 + (char)122 + (char)0 + (char)58 + (char)122 + (char)0 + (char)58;
}
